/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file applications/afs/tools/gnunet-download.c 
 * @brief Main function to download files from GNUnet.
 * @author Christian Grothoff
 **/

#include "gnunet_afs_esed2.h"

static cron_t startTime;

/**
 * Prints the usage information for this command if the user errs.
 * Aborts the program.
 **/
static void printhelp() {
  static Help help[] = {
    { 'a', "anonymity", "LEVEL",
      "set the desired LEVEL of receiver-anonymity" },
    HELP_CONFIG,
    HELP_HELP,
    HELP_HOSTNAME,
    HELP_LOGLEVEL,
    { 'o', "output", "FILENAME",
      "write the file to FILENAME (required)" },
    HELP_VERSION,
    HELP_VERBOSE,
    HELP_END,
  };
  formatHelp("gnunet-download [OPTIONS] GNUNET-URI",
	     "Download file from GNUnet.",
	     help);
}

/**
 * ParseOptions for gnunet-download.
 * @return SYSERR to abort afterwards, OK to continue
 **/
static int parseOptions(int argc,
			char ** argv) {
  int c;  

  while (1) {
    int option_index = 0;
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { "anonymity", 1, 0, 'a' }, 
      { "verbose",   0, 0, 'V' },
      { "output",    1, 0, 'o' },
      { 0,0,0,0 }
    };    
    c = GNgetopt_long(argc,
		      argv, 
		      "a:vhdc:L:H:Vo:", 
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  /* No more flags to process */
    if (YES == parseDefaultOptions(c, GNoptarg))
      continue;
    switch(c) {
    case 'a': {
      unsigned int receivePolicy;
      
      if (1 != sscanf(GNoptarg, "%ud", &receivePolicy)) {
        LOG(LOG_FAILURE,
	    "FAILURE: You must pass a number to the -a option.\n");
	return -1;
      }
      setConfigurationInt("AFS",
      			  "ANONYMITY-RECEIVE",
			  receivePolicy);
      break;
    }
    case 'o':
      FREENONNULL(setConfigurationString("GNUNET-DOWNLOAD",
					 "FILENAME",
					 GNoptarg));
      break;
    case 'v': 
      printf("GNUnet v%s, gnunet-download v%s\n",
	     VERSION,
	     AFS_VERSION);
      return SYSERR;
    case 'V':
      FREENONNULL(setConfigurationString("GNUNET-DOWNLOAD",
					 "VERBOSE",
					 "YES"));
      break;
    case 'h': 
      printhelp(); 
      return SYSERR;
    default: 
      LOG(LOG_FAILURE,
	  "Unknown option %c. Aborting.\n"\
	  "Use --help to get a list of options.\n",
	  c);
      return SYSERR;
    } /* end of parsing commandline */
  } /* while (1) */
  if (argc - GNoptind != 1) {
    LOG(LOG_WARNING, 
	"Not enough arguments. "\
	"You must specify atleast a GNUnet AFS URI\n");
    printhelp();
    return SYSERR;
  }
  FREENONNULL(setConfigurationString("GNUNET-DOWNLOAD",
				     "URI",
				     argv[GNoptind++]));
  return OK;
}

static size_t lastProgress;

/**
 * This method is called whenever data is received.
 * The current incarnation just ensures that the main
 * method exits once the download is complete.
 **/
static void progressModel(ProgressStats * stats,
			  void * data) {
  static unsigned int prevfilesize = 0;

  Semaphore * sem = data;
  if (YES == testConfigurationString("GNUNET-DOWNLOAD",
				     "VERBOSE",
				     "YES")) {
    if (stats->progress != prevfilesize)
      printf("Download at %8u out of %8u bytes (%8.3f kbps)\r",
  	     (unsigned int) stats->progress, 
	     (unsigned int) stats->filesize,
	     (stats->progress/1024.0) / 
	     (((double)(cronTime(NULL)-(startTime-1))) / (double)cronSECONDS) );
    prevfilesize = stats->progress;
  }
  lastProgress = stats->progress;
  if (stats->progress == stats->filesize)
    SEMAPHORE_UP(sem);
}

/**
 * the main function to insert files into GNUnet.
 * @param argc number of arguments from the command line
 * @param argv command line arguments
 * @return return value from insertFile: 0: ok, -1: file not found 
 **/   
int main(int argc, char ** argv) {
  FileIdentifier * fid;
  char * fstring;
  char * fileName;
  Semaphore * doneSem;
  RequestManager * rm;
  size_t length;

  if (SYSERR == initUtil(argc, argv, &parseOptions))
    return 0;

  cronTime(&startTime);
  fileName = getConfigurationString("GNUNET-DOWNLOAD",
				    "FILENAME");
  if (fileName == NULL) {
    LOG(LOG_ERROR,
	"ERROR: You must specify a filename (option -o).\n");
    printhelp();
    return -1;
  }
  fstring = getConfigurationString("GNUNET-DOWNLOAD",
  				   "URI");
  fid = stringToFileIdentifier(fstring);
  FREE(fstring);
  if(fid == NULL) {
    LOG(LOG_ERROR,
        "ERROR: Can't proceed without valid URI.\n");
    return -1;
  }
  length = fid->file_length;
  
  startCron();
  initAnonymityPolicy(NULL);
  doneSem = SEMAPHORE_NEW(0);
  /* insert request for root-node here! */
  rm = downloadFile(fid,
		    fileName, 
		    &progressModel,
		    doneSem);
  if (rm == NULL)
    errexit("FATAL: download failed (error messages should have been provided).\n");
  FREE(fileName); 
  /* wait here until callback indicates that the file
     is complete: */
  SEMAPHORE_DOWN(doneSem);
  LOG(LOG_DEBUG,
      "DEBUG: download %s\n",
      (length == lastProgress) ?
      "complete" : "incomplete");
  printf("Speed was %8.3f kilobyte per second.\n",
	 (lastProgress/1024.0) / 
	 (((double)(cronTime(NULL)-startTime)) / (double)cronSECONDS) );
  FREE(fid);

  SEMAPHORE_FREE(doneSem);
  doneAnonymityPolicy();
  destroyRequestManager(rm);
  stopCron();
  doneUtil();
  
  if (length == lastProgress)
    return 0;
  else
    return 1;
}

/* end of gnunet-download.c */
